#! /usr/bin/env python
# -*- coding: utf-8 -*-

# Version 0.9
# Display and edit metadata from audio files, and rename the files.
# Support mp3, oggvorbis and flac format.
# Depends on PyQt4 and python-tagpy (bindings for taglib)

# Copyright: (c) 2009 Michel Beaulieu <michao@cynoradio.net>
#
# This program is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of
# the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful
# , but WITHOUT ANY WARRANTY; without even the implied warranty
# of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
# the GNU General Public License for more details.

# You should have received a copy of the GNU General Public
# License along with this program.  If not, see
# <http://www.gnu.org/licenses/>.

import sys
import os
from PyQt4 import QtGui,QtCore

# Translations available: en, fr, es, de, ru, pl
Strings = {"Artist":u"Artist", "Title":u"Title", "Album":u"Album", "Genre":u"Genre", "Year":u"Year", "Track":u"Track", "Duration":u"Duration", "Format":u"Format", "Nothing":u"No metadata available", "File":u"File", "Rename":u"Rename", "Open":u"&Open", "Save":u"&Save", "Close":u"&Close", "ErrorMsg":u"Error: unable to read file"}
lang=os.environ['LANG']
if lang[0:2]=='fr':
    Strings = {"Artist":u"Artiste", "Title":u"Titre", "Album":u"Album", "Genre":u"Genre", "Track":u"Piste", "Year":u'Année', "Duration":u"Durée", "Format":u"Format", "Nothing":u'Aucunes métadonnées disponibles', "File":u"Fichier", "Rename":u"Renommer", "Open":u"&Ouvrir", "Save":u"&Sauvegarder", "Close":u"&Fermer", "ErrorMsg":u"Erreur: impossible de lire le fichier"}
elif lang[0:2]=='es':
    Strings = {"Artist":u"Artista", "Title":u"Titulo", "Album":u"Album", "Genre":u'Género', "Track":u"Pista", "Year":u'Año', "Duration":u"Duración", "Format":u"Formato", "Nothing":u"No metadata", "File":u"Archivo", "Rename":u"Cambiar el nombre de", "Open":u"&Abrir", "Save":u"&Salvar", "Close":u"&Cerrar", "ErrorMsg":u"Error: no puede leer el archivo"}
elif lang[0:2]=='de':
    Strings = {"Artist":u'Künstler', "Title":u"Titel", "Album":u"Album", "Genre":u"Genre", "Track":u"Titelnummer", "Year":u"Jahr", "Duration":u"Dauer", "Format":u"Format", "Nothing":u"Keine Metadaten", "File":u"Datei", "Rename":u"Umbenennen", "Open":u"&Öffnen", "Save":u"&Sichern", "Close":u"S&chließen", "ErrorMsg":u"Fehler: die Datei nicht lesen"}
elif lang[0:2]=='ru':
    Strings = {"Artist":u'Исполнитель', "Title":u'Заголовок', "Album":u'Альбом', "Genre":u'Жанр', "Year":u'Год', "Track":u'Дорожка', "Duration":u"Продолжительность", "Format":u"форматы", "Nothing":u'Метаданные недоступны', "File":u"Файл", "Rename":u"Переименовать", "Open":u"&Открывать", "Save":u"&Сохранить", "Close":u"&Закрыть", "ErrorMsg":u"Ошибка: не удалось прочитать файл"}
elif lang[0:2]=='pl':
    Strings = {"Artist":u"Artysta", "Title":u"Tytuł", "Album":u"Album", "Genre":u"Gatunek", "Year":u"Rok", "Track":u"Ścieżka", "Duration":u"Czas", "Format":u"Format", "Nothing":u"Brak danych", "File":u"Plik", "Rename":u"Zmień nazwę", "Open":u"&Otwórz", "Save":u"&Zapisz", "Close":u"Z&amknij", "ErrorMsg":u"Błąd: Nie można odczytać pliku"}

class AudioFile():
    u"""Abstract of an audio file ref from tagpy"""
    def __init__(self,fileinfo):
        u"""fileinfo is an instance of QFileInfo"""
        self.Infos = {}
        self.Tags = {}
        self.Fileinfo=fileinfo
        self.Fullname = fileinfo.absoluteFilePath()
        self.Filename = fileinfo.fileName()
        try:
            import tagpy
            self.File = tagpy.FileRef(str(self.Fullname.toLocal8Bit()))
            ft = self.File.tag()
            ap = self.File.audioProperties()
            if isinstance(self.File.file(),tagpy.mpeg.File):
                self.Infos['Format'] = "MP3"
            elif isinstance(self.File.file(),tagpy.flac.File):
                self.Infos['Format'] = "Flac"
            elif isinstance(self.File.file(),tagpy.ogg.File):
                self.Infos['Format'] = "Ogg"
            self.Tags = {"Artist":ft.artist, "Title":ft.title, "Album":ft.album, "Genre":ft.genre, "Year":ft.year, "Track":ft.track}
            self.Infos["Duration"] = ap.length
            self.Infos["Bitrate"] = ap.bitrate
        except:
            print "%s: %s" % sys.exc_info()[:2]
            
    def saveTags(self,tags):
        for tag in tags.keys():
            if tag=='Artist':
                self.File.tag().artist = tags[tag]
                self.Tags['Artist'] = tags[tag]
            if tag=='Album':
                self.File.tag().album = tags[tag]
                self.Tags['Album'] = tags[tag]
            if tag=='Title':
                self.File.tag().title = tags[tag]
                self.Tags['Title'] = tags[tag]
            if tag=='Track':
                self.File.tag().track = int(tags[tag] or 0)
                self.Tags['Track'] = tags[tag]
            if tag=='Year':
                self.File.tag().year = int(tags[tag] or 0)
                self.Tags['Year'] = tags[tag]
            if tag=='Genre':
                self.File.tag().genre = tags[tag]
                self.Tags['Genre'] = tags[tag]
        self.File.save()
        self.Fileinfo = QtCore.QFileInfo(self.Fileinfo.absoluteFilePath())
        self.Fullname = self.Fileinfo.absoluteFilePath()
        self.Filename = self.Fileinfo.fileName()
        
    def renameFile(self,text):
        text.replace("%a",QtCore.QString(self.Tags['Artist']))
        text.replace("%t",QtCore.QString(self.Tags['Title']))
        text.replace("%b",QtCore.QString(self.Tags['Album']))
        text.replace("%n",QtCore.QString(self.Tags['Track']))
        text.replace("%y",QtCore.QString(self.Tags['Year']))
        text.replace("%g",QtCore.QString(self.Tags['Genre']))
        f = QtCore.QFile(self.Fullname)
        n = QtCore.QStringList([self.Fileinfo.absolutePath(),QtCore.QDir.separator(),text,".",self.Fileinfo.suffix()]).join("")
        f.rename(n)
        self.Fileinfo = QtCore.QFileInfo(n)
        self.Fullname = self.Fileinfo.absoluteFilePath()
        self.Filename = self.Fileinfo.fileName()
        
class Display(QtGui.QWidget):
    u"""Main window"""
    tags = {"Artist":u"", "Title":u"", "Album":u"", "Genre":u"", "Year":u"", "Track":u""}
    Audiofile = AudioFile
    
    def __init__(self, parent=None):
        QtGui.QWidget.__init__(self, parent)
        gridBox = QtGui.QGridLayout()
        
        # Title
        headBox = QtGui.QHBoxLayout()
        self.headLabel = QtGui.QLabel("",self)
        self.headLabel.setAlignment( QtCore.Qt.AlignCenter )
        headBox.addWidget(self.headLabel)

        # Tags and Infos widgets
        artistLabel = QtGui.QLabel(Strings['Artist'])
        self.artistEdit = QtGui.QLineEdit("")
        gridBox.addWidget(artistLabel, 0, 0)
        gridBox.addWidget(self.artistEdit, 0, 1)
        titleLabel = QtGui.QLabel(Strings['Title']) 
        self.titleEdit = QtGui.QLineEdit("")
        gridBox.addWidget(titleLabel, 1, 0)
        gridBox.addWidget(self.titleEdit, 1, 1)
        albumLabel = QtGui.QLabel(Strings['Album']) 
        self.albumEdit = QtGui.QLineEdit("")
        gridBox.addWidget(albumLabel, 2, 0)
        gridBox.addWidget(self.albumEdit, 2, 1)
        trackLabel = QtGui.QLabel(Strings['Track']) 
        self.trackEdit = QtGui.QLineEdit("")
        gridBox.addWidget(trackLabel, 3, 0)
        gridBox.addWidget(self.trackEdit, 3, 1)
        yearLabel = QtGui.QLabel(Strings['Year'])
        self.yearEdit = QtGui.QLineEdit("")
        gridBox.addWidget(yearLabel, 4, 0)
        gridBox.addWidget(self.yearEdit, 4, 1)
        genreLabel = QtGui.QLabel(Strings['Genre'])
        self.genreEdit = QtGui.QLineEdit("")
        gridBox.addWidget(genreLabel, 5, 0)
        gridBox.addWidget(self.genreEdit, 5, 1)
        durationLabel = QtGui.QLabel(Strings['Duration'])
        self.durationEdit = QtGui.QLineEdit("")
        self.durationEdit.setReadOnly(True)
        gridBox.addWidget(durationLabel, 6, 0)
        gridBox.addWidget(self.durationEdit, 6, 1)
        formatLabel = QtGui.QLabel(Strings['Format'])
        self.formatEdit = QtGui.QLineEdit("")
        self.formatEdit.setReadOnly(True)
        gridBox.addWidget(formatLabel, 7, 0)
        gridBox.addWidget(self.formatEdit, 7, 1)

        # File Renaming widget
        renameLabel=QtGui.QLabel(Strings["Rename"])
        self.toggleButton = QtGui.QToolButton()
        self.toggleButton.setArrowType(QtCore.Qt.DownArrow)
        self.fileLabel = QtGui.QLabel(Strings["File"],self)
        self.fileLabel.hide()
        self.fileEdit = QtGui.QLineEdit("",self)
        self.fileEdit.hide()     
        self.newnameLabel = QtGui.QLabel("",self)
        self.newnameLabel.setAlignment( QtCore.Qt.AlignCenter )
        self.newnameLabel.setFrameStyle(QtGui.QFrame.StyledPanel | QtGui.QFrame.Raised )
        self.newnameLabel.hide()
        self.helpLabel=QtGui.QLabel("".join(["%a: ",Strings['Artist'],"   %b: ",Strings["Album"],"   %t: ",Strings["Title"],"\n","%n: ",Strings['Track'],"   %y: ",Strings["Year"],"   %g: ",Strings["Genre"]]))
        self.helpLabel.hide()
        self.helpLabel.setAlignment( QtCore.Qt.AlignCenter )
        # QCompleter does not set isModified() to true !
        # expList= QtCore.QStringList(["%n - %t","%a - %t","%a - %b - %n - %t"])
        # self.completer = QtGui.QCompleter(expList)
        # self.completer.setCompletionMode(QtGui.QCompleter.PopupCompletion)
        # self.fileEdit.setCompleter(self.completer)
        gridBox.addWidget(renameLabel,8,0)
        gridBox.addWidget(self.toggleButton, 8, 1)
        hbox0 = QtGui.QHBoxLayout()
        hbox0.addWidget(self.fileLabel)
        hbox0.addWidget(self.fileEdit)

        # Buttons
        openButton = QtGui.QPushButton(Strings['Open'])
        openButton.setAutoDefault(True)
        self.saveButton = QtGui.QPushButton(Strings['Save'])
        self.saveButton.setAutoDefault(True)
        closeButton = QtGui.QPushButton(Strings['Close'])
        closeButton.setAutoDefault(True)
        hbox1 = QtGui.QHBoxLayout()
        hbox1.addWidget(openButton)
        hbox1.addWidget(self.saveButton)
        hbox1.addWidget(closeButton)

        # Main layout
        self.vbox = QtGui.QVBoxLayout()
        self.vbox.addSpacing(2)
        self.vbox.addLayout(headBox)
        self.vbox.addSpacing(2)
        self.vbox.addLayout(gridBox)
        self.vbox.addLayout(hbox0)
        self.vbox.addWidget(self.newnameLabel)
        self.vbox.addWidget(self.helpLabel)
        self.vbox.insertStretch(-10)
        self.vbox.addSpacing(10)
        self.vbox.addLayout(hbox1)
        self.setLayout(self.vbox)    
        self.resize(300, 150)

        # Connections
        self.connect(closeButton, QtCore.SIGNAL('clicked()'),QtGui.qApp, QtCore.SLOT('quit()') )
        self.connect(self.saveButton, QtCore.SIGNAL('clicked()'), self.dataUpdate )
        self.connect(openButton, QtCore.SIGNAL('clicked()'), self.openFile )
        self.connect(self.toggleButton, QtCore.SIGNAL('clicked()'), self.renameToggle )
        self.connect(self.fileEdit, QtCore.SIGNAL('textChanged(QString)'),self.setNewname)
    #     self.connect(self.completer, QtCore.SIGNAL('activated(QString)'),self.setModified)

    # def setModified(self,text):
    #     self.fileEdit.setText(QtCore.QString(text))
    #     print self.fileEdit.isModified()
    #     self.fileEdit.setModified(True)
    #     print self.fileEdit.isModified()
    
    def setNewname(self,cname):
        name=QtCore.QString(cname)
        name.replace( "%a", self.Audiofile.Tags['Artist'])
        name.replace( "%t", self.Audiofile.Tags['Title'])
        name.replace( "%b", self.Audiofile.Tags['Album'])
        name.replace( "%n", str(self.Audiofile.Tags['Track']))
        name.replace( "%y", str(self.Audiofile.Tags['Year']))
        name.replace( "%g", self.Audiofile.Tags['Genre'])
        self.newnameLabel.setText(name)

    def renameToggle(self):
        if self.fileEdit.isVisible():
            self.fileEdit.hide()
            self.fileLabel.hide()
            self.helpLabel.hide()
            self.newnameLabel.hide()
            self.toggleButton.setArrowType(QtCore.Qt.DownArrow)
            QtCore.QCoreApplication.sendPostedEvents()
            self.adjustSize()
        else:
            self.fileEdit.show()
            self.fileLabel.show()
            self.helpLabel.show()
            self.newnameLabel.show()
            self.toggleButton.setArrowType(QtCore.Qt.UpArrow)
        
    def setFile(self,af):
        self.Audiofile = af
        self.setTags(af.Tags)
        self.setInfos(af.Infos)
        self.headLabel.setText(af.Filename)
        
    def setTags(self,tags):
        self.artistEdit.setText(tags['Artist'])
        self.titleEdit.setText(tags['Title'])
        self.albumEdit.setText(tags['Album'])
        if tags['Track']:
            self.trackEdit.setText(str(tags['Track']))
        else:
            self.trackEdit.setText("")
        if tags['Year']:
            self.yearEdit.setText(str(tags['Year']))
        else:
            self.yearEdit.setText("")
        self.genreEdit.setText(tags['Genre'])

    def setInfos(self,infos):
        if infos['Duration']!="":
            self.durationEdit.setText("%d:%02d"%(divmod(int(infos['Duration']),60)))
        else:
            self.durationEdit.setText("")
        if infos['Format']!="":
            self.formatEdit.setText("".join([str(infos['Format']),", ",str(infos['Bitrate']),"kbps"]))
        else:
            self.formatEdit.setText("")
    
    def dataUpdate(self):
        self.tags['Artist'] = unicode(self.artistEdit.text())
        self.tags['Title'] = unicode(self.titleEdit.text())
        self.tags['Album'] = unicode(self.albumEdit.text())
        self.tags['Track'] = unicode(self.trackEdit.text())
        self.tags['Year'] = unicode(self.yearEdit.text())
        self.tags['Genre'] = unicode(self.genreEdit.text())
        self.Audiofile.saveTags(self.tags)
        if self.fileEdit.isModified():
            self.Audiofile.renameFile(self.fileEdit.text())
            self.setFile(self.Audiofile)
        self.fileEdit.setText("")


    def openFile(self):
        dialog = QtGui.QFileDialog(self);
        dialog.setFileMode(QtGui.QFileDialog.ExistingFile);
        if self.Audiofile:
            dialog.setDirectory(self.Audiofile.Fileinfo.absolutePath())
        fn = dialog.getOpenFileName(self,"","","Audio (*.ogg *.flac *.mp3)")
        if fn:
            try:
                audioFile = AudioFile(QtCore.QFileInfo(fn))
                self.setFile(audioFile)
            except:
                t = {"Artist":u"", "Title":u"", "Album":u"", "Genre":u"", "Year":u"", "Track":u""}
                a = {"Duration":u"", "Format":u"", "Bitrate":u""}
                self.setTags(t)
                self.setInfos(a)
                self.filename.setText("")
                msgBox = QtGui.QMessageBox(self)
                msgBox.setText(Strings['ErrorMsg'])
                msgBox.show()
    
def main(argv=None):
    if argv is None:
        argv = sys.argv
    try:
        app = QtGui.QApplication(argv)
        qb = Display()
        qb.setWindowFlags(QtCore.Qt.Dialog)
        qb.setWindowTitle("AudioMetaData")
        QtCore.QTextCodec.setCodecForCStrings(QtCore.QTextCodec.codecForLocale())
        if len(argv)>1:
            fn=QtCore.QString(argv[1])
            qb.Fileinfo=QtCore.QFileInfo(fn)
            try:
                audioFile = AudioFile(qb.Fileinfo)
                qb.setFile(audioFile)
            except:
                t = {"Artist":u"", "Title":u"", "Album":u"", "Genre":u"", "Year":u"", "Track":u""}
                a = {"Duration":u"", "Format":u"", "Bitrate":u""}
                qb.setTags(t)
                qb.setInfos(a)
                msgBox = QtGui.QMessageBox(qb)
                msgBox.setText(Strings['ErrorMsg'])
                msgBox.show()
        qb.show()
        app.exec_()
        return 0
    except Exception, err:
        print "%s: %s" % sys.exc_info()[:2]
        return 1

if __name__ == "__main__":
    sys.exit(main())
